# Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.

"""
This script holds all the commands for WebCenter application import/export
@author John Whiting
"""
import ora_mbs
import jarray
import WebCenterHandlerHelper as helper
import CommunityHandler as communityHandler
import javax.management as management
from javax.management import ObjectName
from java.lang import Object
from java.lang import String
from java.lang import Boolean
from java.lang import Thread
from java.lang import System
from java.lang import Exception
from java.io import FileOutputStream
from java.io import FileInputStream
from types import *
# Code for detecting syntax error, especially missing colons
def lifecycleCheck():
    print 'File not broken'

# Upgrade WebCenter Application command
def upgradeWebCenterApplication(appName, server='',applicationVersion=''):
    _doUpgrade(appName, server, applicationVersion)

# Export WebCenter Application command
def exportWebCenterApplication(appName, fileName, exportCustomizations=1, exportSecurity=1, exportData=1,server='',applicationVersion=''):
    _doExport(appName, fileName, None, 1, exportCustomizations, 0, exportSecurity, exportData, 1, 0, 1, None,server,applicationVersion)

# Import WebCenter Application command
def importWebCenterApplication(appName, fileName,server='',applicationVersion=''):
    status = _doImport(appName, fileName,None, None, None, None, None, None, None, server,applicationVersion)
    # Upon sucess the status would be set to 0, otherwise it would be set to -1  
    # Perform refresh GS / SpaceTemplate cache explicitly for App Import
    if status == 0:
        try:
            # Params for RefreshGroupSpaceCache:
            # appName,spaceNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion='')
	    communityHandler.refreshGroupSpaceCache(appName,None,1,'all',1,server,applicationVersion)
        except Exception, ex:
	    helper.wcPrintErrorMessage('LF_ERROR_GS_REFRESH')
	    helper.wcPrintExceptionMessage(ex.getMessage())
	    helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
	    return
	try:
	    # Params for RefreshSpaceTemplateCache:
	    #appName,spaceTemplateNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion=''):
	    communityHandler.refreshSpaceTemplateCache(appName,None,1,'all',1,server,applicationVersion)
        except Exception, ex:
	    helper.wcPrintErrorMessage('LF_ERROR_GS_TEMPLATES_REFRESH')
	    helper.wcPrintExceptionMessage(ex.getMessage())
	    helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
	    return            


# Export Portlet Client Metadata command
def exportPortletClientMetadata(appName, fileName, exportPersonalizations=1, server='',applicationVersion=''):
    _doExport(appName, fileName, None, 1, 1, exportPersonalizations, 0, 0, 0, 0, 1, ['oracle.webcenter.portlet'],server,applicationVersion)

# Import Portlet Client Metadata command
def importPortletClientMetadata(appName, fileName,server='',applicationVersion=''):
    _doImport(appName, fileName,None, None, None, None, None, None, None, server,applicationVersion)

# Export Producer Metadata command (deprecated)
def exportProducerMetadata(appName, fileName, exportPersonalizations=1, server='', applicationVersion=''):
    exportPortletClientMetadata(appName, fileName, exportPersonalizations, server, applicationVersion)

# Import Producer Metadata command (deprecated)
def importProducerMetadata(appName, fileName, server='', applicationVersion=''):
    importPortletClientMetadata(appName, fileName, server, applicationVersion)

# Notification Listener used to output the notification messages from 
# the lifecycle operations
class LifecycleNotificationListener (management.NotificationListener):
    # Constructor
    def __init__(self):
        self.progress = 0
        self.enableOutput = 1

    # Handles the notifications
    def handleNotification(self, notification, handback):
        if self.enableOutput:
            print notification.getMessage()
        self.progress = notification.getUserData()

    # Whether the progress is complete
    def isComplete(self):
        return self.progress == 1.0

    # Wait up to 10 seconds for the progress to complete
    def waitUntilComplete(self):
        try:
            for i in range(10):
                if self.isComplete():
                    return
                Thread.sleep(1000)
        finally:
            self.enableOutput = 0

    # Disable output
    def disableOutput(self):
        self.enableOutput = 0

# No-op Notificiation Listener 
class NoopNotificationListener (management.NotificationListener):
    def handleNotification(self, notification, handback):
        pass

def _doUpgrade(appName, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    try:
        ora_mbs.invoke(lifecycleManagerObjectName, 'doUpgrade', jarray.array([], Object), jarray.array([], String))
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.printStackTrace())
        helper.wcPrintErrorMessage('LF_ERROR_UPGRADE')
        return
    helper.wcPrintMessage('LF_UPGRADE_COMPLETED')


# Performs an export
# This method is also being called from ExpImpGsWlst. 
def _doExport(appName, fileName, scopeNames, exportBaseDocuments, exportCustomizations, exportPersonalizations, exportSecurity, exportData, exportPreferences, forceOffline, exportContentDirectory, serviceIds,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'export') == 0:
        return
    if _validateStringDataType('fileName', fileName, 'export') == 0:
        return
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return
    invalidArgument = 0
    if _validateArgument('LF_INVALID_EXPORTCUSTOMIZATIONS', exportCustomizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTPERSONALIZATIONS', exportPersonalizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTSECURITY', exportSecurity) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTDATA', exportData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTCONTENTDIR', exportContentDirectory) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    try:
        try:
            serviceIdsArray = None
            if serviceIds:
                serviceIdsArray = jarray.array(serviceIds, String)

            scopeNamesArray = None
            # We pass groupSpace/template names as scopeNames while exporting groupspaces/templates 
            # from the webcenter app from ExpImPGsWlst.
            if scopeNames:
                scopeNamesArray = jarray.array(scopeNames, String)

            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([scopeNamesArray, Boolean(exportBaseDocuments),
                           Boolean(exportCustomizations), Boolean(exportPersonalizations), Boolean(exportSecurity), Boolean(exportData),
                           Boolean(exportPreferences), Boolean(forceOffline), Boolean(exportContentDirectory), serviceIdsArray], Object),
                           jarray.array(['[Ljava.lang.String;', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 
                           'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean','java.lang.Boolean','[Ljava.lang.String;'], String))
            objectName = _getInstanceObjectName(instanceName)
            #listener = LifecycleNotificationListener()
            #ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            return
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startExport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            if scopeNames:
                print ''
	    else:
                helper.wcPrintErrorMessage('LF_ERROR_START_EXPORT')
            return
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            return
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            return
        #listener.waitUntilComplete()
        helper.wcPrintMessage('LF_EXPORT_COMPLETED',fileName)
    finally:
        if outputStream:
            outputStream.close()
        if listener:
            listener.disableOutput()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs a site resource export
# This method is being called from ExpImpWlst.
def _doSiteResExport(appName, fileName, scopeName, resourceType, resourceGUID, exportContentDirectory, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'export') == 0:
        return
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    try:
        try:
            resGuidArray = None
            if resourceGUID:
                resGuidArray = jarray.array(resourceGUID, String)

            # Change the external resource type name to internal name
            if resourceType == 'pageTemplate':
                resourceType = 'siteTemplate'
            elif resourceType == 'mashupStyle':
                resourceType = 'taskflowStyle'
            if _validateStringDataType('fileName', fileName, 'export') == 0:
                return
	    if _validateStringDataType('spaceName', scopeName, 'export') == 0:
                return
            if _validateStringDataType('resourceType', resourceType, 'export') == 0:
                return
            if _validateArgument('LF_INVALID_EXPORTCONTENTDIR', exportContentDirectory) == 0:
               return
	    instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([String(scopeName),String(resourceType), resGuidArray, Boolean(exportContentDirectory)], Object), jarray.array(['java.lang.String', 'java.lang.String' , '[Ljava.lang.String;', 'java.lang.Boolean'], String))
            objectName = _getInstanceObjectName(instanceName)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            return
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startExport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_ERROR_START_EXPORT')
            return
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            return
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            return
        helper.wcPrintMessage('LF_EXPORT_COMPLETED',fileName)
    finally:
        if outputStream:
            outputStream.close()
        if listener:
            listener.disableOutput()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs an import of Webcenter App and Portlet Client
def _doImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName,resourceType, server='',applicationVersion=''):
    return _doScopeImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName, resourceType, 0, server, applicationVersion)

# Performs an import
# This method is also being called from ExpImpHandler. 
# This method does an Import of Site resource as well.
def _doScopeImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName,resourceType, overwriteContentDirectory, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'import') == 0:
        return -1
    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return -1
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return -1
    invalidArgument = 0   
    if _validateArgument('LF_INVALID_IMPORTCUSTOMIZATIONS', exportCustomizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_IMPORTSECURITY', exportSecurity) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_IMPORTDATA', exportData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_OVERWRITECONTENTDIR', overwriteContentDirectory) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return -1
    instanceName = None
    objectName = None
    inputStream = None
    listener = None
    try:
        try:
            instanceName = None
            # for portlet client metadata import
            if exportCustomizations is None and exportSecurity is None and exportData is None and scopeName is None and resourceType is None:
                instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([], Object), jarray.array([], String))
            # for site resource import
            elif scopeName is not None and resourceType is not None:
              # Change the external resource type name to internal name
              if resourceType == 'pageTemplate':
                  resourceType = 'siteTemplate'
              elif resourceType == 'mashupStyle':
                  resourceType = 'taskflowStyle'
              if _validateStringDataType('spaceName', scopeName, 'import') == 0:
                  return -1
              if _validateStringDataType('resourceType', resourceType, 'import') == 0:
                  return -1
              instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([String(scopeName), String(resourceType), Boolean(overwriteContentDirectory)], Object), jarray.array(['java.lang.String', 'java.lang.String', 'java.lang.Boolean'], String))
            # for group space import
            elif exportCustomizations is not None and exportSecurity is not None and exportData is not None:
              if parentScope is None:
                 parentScope = ''
              if _validateStringDataType('parentSpace', parentScope, 'import') == 0:
		  return -1
              instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([Boolean(exportCustomizations), Boolean(exportSecurity), Boolean(exportData), Boolean(forceOffline), String(parentScope), Boolean(overwriteContentDirectory)], Object), jarray.array(['java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean', 'java.lang.String', 'java.lang.Boolean'], String))
            objectName = _getInstanceObjectName(instanceName)
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_IMPORT_INSTANCE')
            return -1
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startImport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_START_IMPORT')
            return -1
        try:
            inputStream = FileInputStream(fileName)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE',fileName)
            return -1
        try:
            bytes = _readBytes(inputStream)
            while bytes:
                _putBytes(objectName, streamHandle, bytes)
                bytes = _readBytes(inputStream)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.printStackTrace())
            helper.wcPrintErrorMessage('LF_ERROR_PUT_BYTES')
            return -1
        try:
            ora_mbs.invoke(objectName, 'doImport', jarray.array([streamHandle], Object), jarray.array(['java.lang.String'], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_DO_IMPORT')
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
	    return -1
        helper.wcPrintMessage('LF_IMPORT_COMPLETED',fileName)
	return 0
    finally:
        if inputStream:
            inputStream.close()
        if listener:
            listener.disableOutput()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs an import
def _doImportInternal(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeGUID, resourceType, overwriteContentDirectory, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    lifecycleManagerObjName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjName is None:
        return
    scopeName = None
    try:
        scopeName = ora_mbs.invoke(lifecycleManagerObjName, 'getScopeName', jarray.array([String(scopeGUID)], Object), jarray.array(['java.lang.String'], String))
	_doScopeImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName, resourceType, overwriteContentDirectory, server, applicationVersion)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.printStackTrace())
        helper.wcPrintErrorMessage('LF_ERROR_START_IMPORT')
        return
    

# Gets an array of MBean object names that match the pattern supplied
def _getMBeanArray(pattern):
    objname= ObjectName(pattern)
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
        return mbarray
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('LF_ERROR_MDS_QUERY_NAMES',pattern)
    return None

# Finds an MBean that matches the pattern and contains the search string
def _findMBean(pattern, searchStr):
    mbeanarray = _getMBeanArray(pattern)
    if mbeanarray is not None:
        for i in range(len(mbeanarray)):
            objnamestr = mbeanarray[i].toString()
            matches = 1
            for s in searchStr.split(','):
                if objnamestr.find(s) < 0:
                    matches = 0
                    break
            if (matches == 1):
                return mbeanarray[i]
    return None

# Gets the object name of the LifecycleManager MBean
def _getLifecycleManagerObjectName(appName):
    searchNameStr = 'name=LifecycleManager'
    searchTypeStr = 'type=LifecycleManager'
    startTag = 'oracle.webcenter.lifecycle:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.lifecycle:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'LF_ERROR_LIFECYCLE_MANAGER_MBEAN_NOT_FOUND',startTag)
    # Workaround for bug 7437672 - add/remove a no-op notification listener
    #if objectName is not None:
        #listener = NoopNotificationListener()
        #ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
        #ora_mbs.getMbsInstance().removeNotificationListener(objectName, listener)
    return objectName

# Finds the object name for the import/export instance MBean using the name returned from the LifecycleManager MBean
def _getInstanceObjectName(name):
    [domain,keys]=name.split(':')
    return _findMBean('*' + domain + ':*', keys)


# Destroys the import/export instance MBean
def _destroyInstance(lifecycleManagerObjectName,instanceName):
    try:
        ora_mbs.invoke(lifecycleManagerObjectName, 'destroyInstance', jarray.array([instanceName], Object), jarray.array(['java.lang.String'], String))
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('LF_ERROR_DESTROY_INSTANCE')

# Downloads bytes from the exported archive
def _getBytes(objectName, streamHandle):
    return ora_mbs.invoke(objectName, 'getBytes', jarray.array([streamHandle, 1024 * 128], Object), jarray.array(['java.lang.String', 'int'], String))

# Uploads bytes from the exported archive
def _putBytes(objectName, streamHandle,bytes):
    ora_mbs.invoke(objectName, 'putBytes',jarray.array([streamHandle, bytes], Object), jarray.array(['java.lang.String', '[B'], String))

# Reads bytes from an InputStream
def _readBytes(inputStream):
    length = 1024 * 128
    bytes = jarray.zeros(length, 'b')
    i = inputStream.read(bytes)
    if i < 0:
        return None
    elif i < length:
        newBytes = jarray.zeros(i, 'b')
        System.arraycopy(bytes, 0, newBytes, 0, i)
        return newBytes
    else:
        return bytes

# Clean-up after running the lifecycle operations
def _cleanup(appName,objectName):
    if objectName:
        _destroyInstance(appName, objectName)
    helper.webcenterRestore()

# Validation to check the data type of arguments, PyString check for wsadmin console commands (string passed as PyString type)
def _validateStringDataType(argument, argValue, operation):
    try:
	if type(argValue) is StringType:
            return 1
        else:
	    obtainedType = str(type(argValue))
       	    argIfPythonString = "org.python.core.PyString"
            if cmp(obtainedType,argIfPythonString) == 0:
	        return 1
	    else:
		raise TypeError('Expected DataType not found.')
    except TypeError:
        helper.wcPrintErrorMessage('LF_OPERATION_FAIL', operation)
        argumentInMessage = '\'' + argument + '\''
        helper.wcPrintErrorMessage('LF_MESSAGE_INVALID_ARGUMENT', argumentInMessage)
        invalidValue = '\'' + str(argValue) + '\''
        helper.wcPrintErrorMessage('LF_MESSAGE_INVALID_VALUE', invalidValue)
        return 0

# Checking for value of boolean arguments for export/import to  be 0 or 1 only
def _validateArgument(keyMessage, value):
    if (value == 0) or (value == 1) or (value is None): 
     	return 1
    else:
	invalidValue = '\'' + str(value) + '\''
	helper.wcPrintErrorMessage(keyMessage, invalidValue)
        return 0
